/*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %                     IIIIIIIIII    PPPPPPPP      LL                          %
 %                         II        PP      PP    LL                          %
 %                         II        PP       PP   LL                          %
 %                         II        PP      PP    LL                          %
 %                         II        PPPPPPPP      LL                          %
 %                         II        PP            LL                          %
 %                         II        PP            LL                          %
 %                     IIIIIIIIII    PP            LLLLLLLL                    %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %                   Read/Write Scanalytics IPLab Image Format                 %
 %                                  Sean Burke                                 %
 %                                  2007.08.02                                 %
 %                                     v 0.1                                   %
 %                                                                             %
 %  Copyright 1999-2007 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
 %    http://www.imagemagick.org/script/license.php                            %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
 %  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   %
 %  See the License for the specific language governing permissions and        %
 %  limitations under the License.                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %
 */

/*
 Include declarations.
 */
#include "magick/studio.h"
#include "magick/blob.h"
#include "magick/blob-private.h"
#include "magick/colorspace.h"
#include "magick/exception.h"
#include "magick/exception-private.h"
#include "magick/image.h"
#include "magick/image-private.h"
#include "magick/list.h"
#include "magick/magick.h"
#include "magick/memory_.h"
#include "magick/monitor.h"
#include "magick/quantum.h"
#include "magick/static.h"
#include "magick/string_.h"

/* 
Tyedef declarations
 */

typedef struct _IPLInfo
{
	unsigned long
	tag,
	size,
	time,
	z,
	width,
	height,
	colors,
	depth,
	byteType;
} IPLInfo;

static inline size_t CheckOverflowException(const Image *image,
											const size_t extend)
{
	if ((extend*image->columns) < (size_t) image->columns)
		ThrowMagickFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
								  image->filename);
	return(extend*image->columns);
}

void increase (void *pixel, int byteType){
	switch(byteType){
		case 0:(*((unsigned char *) pixel))++; break;
		case 1:(*((signed int *) pixel))++; break;
		case 2:(*((unsigned int *) pixel))++; break;
		case 3:(*((signed long *) pixel))++; break;
		default:(*((unsigned int *) pixel))++; break;
	}	
}

/*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %   I s I P L                                                                 %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  IsIPL() returns MagickTrue if the image format type, identified by the
 %  magick string, is IPL.
 %
 %  The format of the IsIPL method is:
 %
 %      MagickBooleanType IsIPL(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
 %    o magick: This string is generally the first few bytes of an image file
 %      or blob.
 %
 %    o length: Specifies the length of the magick string.
 %
 %
 */
static MagickBooleanType IsIPL(const unsigned char *magick,const size_t length)
{
	if (length < 4)
		return(MagickFalse);
	if (LocaleNCompare((char *) magick,"iiii",4) == 0)
		return(MagickTrue);
	return(MagickFalse);
}

/*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %    R e a d I P L I m a g e                                                  %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  ReadIPLImage() reads a Scanalytics IPLab image file and returns it.  It 
 %  allocates the memory necessary for the new Image structure and returns a 
 %  pointer to the new image.
 %
 %  According to the IPLab spec, the data is blocked out in five dimensions:
 %  { t, z, c, y, x }.  When we return the image, the latter three are folded
 %  into the standard "Image" structure.  The "scenes" (image_info->scene) 
 %  correspond to the order: { {t0,z0}, {t0, z1}, ..., {t1,z0}, {t1,z1}... }
 %  The number of scenes is t*z.
 %
 %  The format of the ReadIPLImage method is:
 %
 %      Image *ReadIPLImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
 %    o image_info: The image info.
 %
 %    o exception: return any errors or warnings in this structure. 
 %
 %
 */

static Image *ReadIPLImage(const ImageInfo *image_info,ExceptionInfo *exception)
{
	
	/* 
	Declare variables 
	 */
	Image *image;
	MagickBooleanType status;
	long y,c;
	register long x;
	register PixelPacket *q;
	//register unsigned char *p;
	unsigned char magick[12];
	ssize_t count;
	unsigned long t_count=0;
	unsigned long pixel;
	size_t length;
	IPLInfo
		ipl_info;
	/*
	 Open Image
	 */
	
	assert(image_info != (const ImageInfo *) NULL);
	assert(image_info->signature == MagickSignature);
	if ( image_info->debug != MagickFalse)
		(void) LogMagickEvent(TraceEvent, GetMagickModule(), "%s",
							  image_info->filename);
	assert(exception != (ExceptionInfo *) NULL);
	assert(exception->signature == MagickSignature);
	image=AllocateImage(image_info);
	status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
	if (status == MagickFalse)
	{
		image=DestroyImageList(image);
		return((Image *) NULL);
	}
	
	/*
	 Read IPL image
	 */
	/*  Set default resolution */
	image->x_resolution=1;
	image->y_resolution=1;
	/* 
		Determine endianness 
	 If we get back "iiii", we have LSB,"mmmm", MSB
	 */
	count=ReadBlob(image,4,magick); // header count: 4
	if((LocaleNCompare((char *) magick,"iiii",4) == 0))  // iiii
		image->endian=LSBEndian;
	else{
		if((LocaleNCompare((char *) magick,"mmmm",4) == 0)) // mmmm
			image->endian=MSBEndian;
		else{
			ThrowReaderException(CorruptImageError, "ImproperImageHeader");
		}
	}
	
	/* Skip o'er the next 8 bytes (garbage) */
	count=ReadBlob(image, 8, magick); // header count: 12
	/*
	 Excellent, now we read the header unimpeded.
	 */
	count=ReadBlob(image,4,magick); // header count: 16
	if((LocaleNCompare((char *) magick,"data",4) != 0))  // "data"
		ThrowReaderException(CorruptImageError, "ImproperImageHeader");
	ipl_info.size=ReadBlobLong(image); // header count: 20
	ipl_info.width=ReadBlobLong(image); // header count: 24
	ipl_info.height=ReadBlobLong(image); // header count: 28
	if((ipl_info.width == ~0UL) || (ipl_info.height == ~0UL))
		ThrowReaderException(CorruptImageError, "ImproperImageHeader");
	ipl_info.colors=ReadBlobLong(image); // header count: 32
	if(ipl_info.colors == 3){ image->colorspace=RGBColorspace;}
	else { image->colorspace = GRAYColorspace; }
	ipl_info.z=ReadBlobLong(image); // header count: 36
	ipl_info.time=ReadBlobLong(image); // header count: 40
	
	ipl_info.byteType=ReadBlobLong(image); // header count: 44
	
	switch (ipl_info.byteType) {
		case 0: ipl_info.depth=8;break;
		case 1: ipl_info.depth=16;break;
		case 2: ipl_info.depth=16;break;
		case 3: ipl_info.depth=32;break;
		default: ipl_info.depth=16;break;
	}
	
	// number of bytes/line is # colors*depth/8
	length=CheckOverflowException(image, (ipl_info.colors)*ipl_info.depth/8);
	
	/* Thats all we need if we are pinging. */
	if (image_info->ping != MagickFalse)
	{
		CloseBlob(image);
		return(GetFirstImageInList(image));
	}
	do
	{
		/* 
		Covert IPL binary to pixel packets
		 */
		image->columns=ipl_info.width;
		image->rows=ipl_info.height;
		image->depth=ipl_info.depth; 
		for (c=0; c < (long) ipl_info.colors; c++){
			for (y=0; y < (long) image->rows; y++){
				q=SetImagePixels(image, 0, y, image->columns, 1);
				if (q == (PixelPacket *) NULL)
					break;
				for (x=0; x < (long) image->columns; x++){
					switch(ipl_info.byteType){
						case 0: pixel=ReadBlobByte(image); break;
						case 1: pixel=ReadBlobShort(image); break;
						case 2: pixel=ReadBlobShort(image); break;
						case 3: pixel=ReadBlobLong(image); break;
						default: pixel=ReadBlobShort(image); break;
					}
					if(ipl_info.colors == 3){
						// avs method doesn't work need to use psd pixel by pixel method
						switch(c){
							case 1:
							{
								q->green=ScaleAnyToQuantum(pixel, (1UL << (image->depth))-1);
								break;
							}
							case 2:
							{
								q->blue=ScaleAnyToQuantum(pixel, (1UL << (image->depth))-1);
								break;
							}
							default:
							{
								q->red=ScaleAnyToQuantum(pixel, (1UL << (image->depth))-1);
								break;
							}
						}
					}
					else {
						q->red=ScaleAnyToQuantum(pixel, (1UL << (image->depth))-1);
						q->green=ScaleAnyToQuantum(pixel, (1UL << (image->depth))-1);
						q->blue=ScaleAnyToQuantum(pixel, (1UL << (image->depth))-1); 
					}
					q++;
					if (SyncImagePixels(image) == MagickFalse)
						break;
					if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
						(QuantumTick(y,image->rows) != MagickFalse)){
						status=image->progress_monitor(LoadImageTag,y,image->rows,
													   image->client_data);
						if (status == MagickFalse)
							break;
					}     
				} // x for loop
			} // y for loop
		} // c for loop	
		t_count++;
		if (EOFBlob(image) != MagickFalse)
		{
			ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
							   image->filename);
			break;
		}
		/*
		 Proceed to next image.
		 */
		if (image_info->number_scenes != 0)
			if (image->scene >= (image_info->scene+image_info->number_scenes-1))
				break;	
		/*
		 Since IPL specifies no frame to frame delimiter, we are going to have to 
		 skip the sanity check.
		 */
		
		AllocateNextImage(image_info, image);
		if (GetNextImageInList(image) == (Image *) NULL)
		{
			image=DestroyImageList(image);
			return((Image *) NULL);
		}
		image=SyncNextImageInList(image);
		if (image->progress_monitor != (MagickProgressMonitor) NULL)
		{
			status=image->progress_monitor(LoadImagesTag,TellBlob(image),
										   GetBlobSize(image),image->client_data);
			if (status == MagickFalse)
				break;
		}
	} while (t_count < ipl_info.z*ipl_info.time);
	CloseBlob(image);
	return(GetFirstImageInList(image));
}

/*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %   W r i t e I P L I m a g e                                                 %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  WriteIPLImage() writes an image to a file in Scanalytics IPLabimage format.
 %
 %  The format of the WriteIPLImage method is:
 %
 %      MagickBooleanType WriteIPLImage(const ImageInfo *image_info,Image *image)
 %
 %  A description of each parameter follows.
 %
 %    o image_info: The image info.
 %
 %    o image:  The image.
 %
 %
 */

static MagickBooleanType WriteIPLImage(const ImageInfo *image_info,Image *image)
{
	MagickBooleanType
    status;
	
	MagickOffsetType
		scene;
	
	register const PixelPacket
		*p;
	
	unsigned long
		x, y, z;
	
	register unsigned char
		*q_c;
	unsigned char
		*pixels_c, *pixels_s;
	
	IPLInfo
		ipl_info;
	
	/*
	 Open output image file.
	 */
	assert(image_info != (const ImageInfo *) NULL);
	assert(image_info->signature == MagickSignature);
	assert(image != (Image *) NULL);
	assert(image->signature == MagickSignature);
	if (image->debug != MagickFalse)
		(void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
	status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
	if (status == MagickFalse)
		return(status);
	scene=0;
	
	switch(image->depth){
		case 8: ipl_info.byteType = 0;break;
		default: ipl_info.byteType = 2; break;
	}
	
	ipl_info.z = GetImageListLength(image);
	/* There is no current method fo detecting whether we have T or Z stacks */
	ipl_info.time = 1;
	ipl_info.width = image->columns;
	ipl_info.height = image->rows;
	
    if (image->colorspace == UndefinedColorspace)
		(void) SetImageColorspace(image,RGBColorspace);
	
	if(image->colorspace == RGBColorspace) { ipl_info.colors = 3; }
	else{ ipl_info.colors = 1; }
	
	ipl_info.size = 28 + 
		((image->depth)/8)*ipl_info.height*ipl_info.width*ipl_info.colors*ipl_info.z;
	
	/* Ok!  Calculations are done.  Lets write this puppy down! */
	
	/*
	 Write IPL header.
	 */
	WriteBlob(image, 4, (unsigned char *) "iiii");
	WriteBlobLSBLong(image, 4);
	WriteBlob(image, 4, (unsigned char *) "100f");
	WriteBlob(image, 4, (unsigned char *) "data");
	WriteBlobLSBLong(image, ipl_info.size);
	WriteBlobLSBLong(image, ipl_info.width);
	WriteBlobLSBLong(image, ipl_info.height);
	WriteBlobLSBLong(image, ipl_info.colors);
	WriteBlobLSBLong(image, ipl_info.z);
	WriteBlobLSBLong(image, ipl_info.time);
	WriteBlobLSBLong(image, ipl_info.byteType);
	
	if(ipl_info.byteType == 0) {
		do
		{
			/*
			 Allocate memory for pixels.
			 */
			pixels_c=(unsigned char *)
			AcquireMagickMemory((size_t) image->columns*sizeof(PixelPacket));
			if (pixels_c == (unsigned char *) NULL)
				ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
			/*
			 Convert MIFF to IPL raster pixels.
			 */
			for(z = 0; z < ipl_info.z; z++){
				/* Red frame */
				for(y = 0; y < ipl_info.height; y++){
					p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
					if (p == (PixelPacket *) NULL)
						break;
					q_c=pixels_c;
					for(x = 0; x < ipl_info.width; x++){
						*q_c++ = ScaleQuantumToChar(p->red);
						p++;
					}
					(void) WriteBlob(image,(size_t) (q_c-pixels_c),pixels_c);
					if (image->previous == (Image *) NULL){
						if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
							(QuantumTick(y,image->rows) != MagickFalse))
						{
							status=image->progress_monitor(SaveImageTag,y,image->rows,
														   image->client_data);
							if (status == MagickFalse)
								break;
						}
					}
				}
				if(ipl_info.colors == 3){
					/* Green frame */
					for(y = 0; y < ipl_info.height; y++){
						p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
						if (p == (PixelPacket *) NULL)
							break;
						q_c=pixels_c;
						for(x = 0; x < ipl_info.width; x++){
							*q_c++ = ScaleQuantumToChar(p->green); 
							p++;
						}
						(void) WriteBlob(image,(size_t) (q_c-pixels_c),pixels_c);
						if (image->previous == (Image *) NULL){
							if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
								(QuantumTick(y,image->rows) != MagickFalse))
							{
								status=image->progress_monitor(SaveImageTag,y,image->rows,
															   image->client_data);
								if (status == MagickFalse)
									break;
							}
						}
					}
					/* Blue frame */
					for(y = 0; y < ipl_info.height; y++){
						p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
						if (p == (PixelPacket *) NULL)
							break;
						q_c=pixels_c;
						for(x = 0; x < ipl_info.width; x++){
							*q_c++ = ScaleQuantumToChar(p->blue);
							p++;
						}
						(void) WriteBlob(image,(size_t) (q_c-pixels_c),pixels_c);
						if (image->previous == (Image *) NULL){
							if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
								(QuantumTick(y,image->rows) != MagickFalse))
							{
								status=image->progress_monitor(SaveImageTag,y,image->rows,
															   image->client_data);
								if (status == MagickFalse) break;
							} // if
						} // if
					} // for y
				} // if colors == 3
			} // for z
			pixels_c=(unsigned char *) RelinquishMagickMemory(pixels_c);
			if (GetNextImageInList(image) == (Image *) NULL)
				break;
			image=SyncNextImageInList(image);
			if (image->progress_monitor != (MagickProgressMonitor) NULL)
			{
				status=image->progress_monitor(SaveImagesTag,scene,
											   GetImageListLength(image),image->client_data);
				if (status == MagickFalse)
					break;
			}
			scene++;
			
		}while (image_info->adjoin != MagickFalse);
	} // if depth == char
	else {
		do
		{
			/*
			 Allocate memory for pixels.
			 */
			pixels_s=(unsigned char *)
			AcquireMagickMemory((size_t) 2*image->columns*sizeof(PixelPacket));
			if (pixels_s == (unsigned char *) NULL)
				ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
			/*
			 Convert MIFF to IPL raster pixels.
			 */
			for(z = 0; z < ipl_info.z; z++){
				/* Red frame */
				for(y = 0; y < ipl_info.height; y++){
					p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
					if (p == (PixelPacket *) NULL)
						break;
					for(x = 0; x < ipl_info.width; x++){
						WriteBlobLSBShort(image, ScaleQuantumToShort(p->red));
						p++;
					}
			if (image->previous == (Image *) NULL){
						if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
							(QuantumTick(y,image->rows) != MagickFalse))
						{
							status=image->progress_monitor(SaveImageTag,y,image->rows,
														   image->client_data);
							if (status == MagickFalse)
								break;
						}
					}
				}
				if(ipl_info.colors == 3){
					/* Green frame */
					for(y = 0; y < ipl_info.height; y++){
						p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
						if (p == (PixelPacket *) NULL)
							break;
						for(x = 0; x < ipl_info.width; x++){
							WriteBlobLSBShort(image, ScaleQuantumToShort(p->green));
							p++;
						}
						if (image->previous == (Image *) NULL){
							if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
								(QuantumTick(y,image->rows) != MagickFalse))
							{
								status=image->progress_monitor(SaveImageTag,y,image->rows,
															   image->client_data);
								if (status == MagickFalse)
									break;
							}
						}
					}
					/* Blue frame */
					for(y = 0; y < ipl_info.height; y++){
						p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
						if (p == (PixelPacket *) NULL)
							break;
						for(x = 0; x < ipl_info.width; x++){
							WriteBlobLSBShort(image, ScaleQuantumToShort(p->blue));
							p++;
						}
						if (image->previous == (Image *) NULL){
							if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
								(QuantumTick(y,image->rows) != MagickFalse))
							{
								status=image->progress_monitor(SaveImageTag,y,image->rows,
															   image->client_data);
								if (status == MagickFalse) break;
							} // if
						} // if
					} // for y
				} // if colors == 3
			} // for z
			pixels_s=(unsigned char *) RelinquishMagickMemory(pixels_s);
			if (GetNextImageInList(image) == (Image *) NULL)
				break;
			image=SyncNextImageInList(image);
			if (image->progress_monitor != (MagickProgressMonitor) NULL)
			{
				status=image->progress_monitor(SaveImagesTag,scene,
											   GetImageListLength(image),image->client_data);
				if (status == MagickFalse)
					break;
			}
			scene++;			
		}while (image_info->adjoin != MagickFalse);
	} // else
	WriteBlob(image, 4, (unsigned char *) "fini");
	WriteBlobLSBLong(image, 0);
	
	CloseBlob(image);
	return(MagickTrue);
}


/*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %   R e g i s t e r I P L I m a g e                                           %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 % RegisterIPLImage() add attributes for the Scanalytics IPL image format to the
 % list of supported formats.  
 %
 */
ModuleExport void RegisterIPLImage(void)
{
	MagickInfo
    *entry;
	
	entry=SetMagickInfo("IPL");
	entry->decoder=(DecoderHandler *) ReadIPLImage;
	entry->encoder=(EncoderHandler *) WriteIPLImage;
	entry->magick=(MagickHandler *) IsIPL;
	entry->adjoin=MagickFalse;
	entry->description=ConstantString("IPL Image Sequence");
	entry->module=ConstantString("IPL");
	(void) RegisterMagickInfo(entry);
}

/*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %   U n r e g i s t e r I P L I m a g e                                       %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  UnregisterIPLImage() removes format registrations made by the
 %  IPL module from the list of supported formats.
 %
 %  The format of the UnregisterIPLImage method is:
 %
 %      UnregisterIPLImage(void)
 %
 */
ModuleExport void UnregisterIPLImage(void)
{
	(void) UnregisterMagickInfo("IPL");
}

